/*
 * 
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif
#include "trace.h"
#include "level.h"
#include "image.h"
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <SDL.h>

static struct t_level *Level_Load_Real(char *filename);


/*
 *
 *
 */


static struct t_level *Level_Load_Real(char *filename)
{
  struct level_desc lvl;
  struct t_level *level;
  int file;
  int nsquares, i, j, x, y, *map;
  char bg_filename[1024];

  file=open(filename,O_RDONLY);
  if (file==-1)
    return NULL;

  /* Load the header file */
  
  read(file,&lvl,sizeof(struct level_desc));
  if (!(lvl.version[0] == 'L' && lvl.version[1] == 'V' && lvl.version[2] == 'L' && lvl.version[3] == '1'))
   {
     close(file);
     return NULL;
   }

  level = malloc(sizeof(struct t_level));
  if (!level)
   {
     close(file);
     return NULL;
   }

  level->sizex = ntohs(*(unsigned short int *)lvl.sizex);
  level->sizey = ntohs(*(unsigned short int *)lvl.sizey);

  level->nsquares = nsquares = ntohs(*(unsigned short int *)lvl.nsquares);

  /* Load the background filename */
  strcpy(bg_filename,"backgrounds/");
  i=strlen(bg_filename); 
  while(i<1023)
   {
     read(file,bg_filename+i,1);
     trace("%c",bg_filename[i]);
     if (bg_filename[i]==0)
       break;
     i++;
   }
  bg_filename[i]=0;
  if (i==strlen("backgrounds/"))
   {
     level->background = NULL;
   }
  else
   {
     level->background = Image_Load(bg_filename);
   }

  /* Load the data squares */
  level->squares = malloc(nsquares * sizeof(struct t_square));
  if (level->squares==NULL)
   {
     close(file);
     free(level);
     return NULL;
   }
  memset(level->squares,0,nsquares * sizeof(struct t_square));

  for (i=1;i<nsquares;i++)
   {
     int blksz;
     struct square_desc s;
     char *images, *p;
     read(file,&s,sizeof(struct square_desc));
     level->squares[i].properties = ntohl(*(unsigned int *)s.properties);
     level->squares[i].nimages = ntohs(*(unsigned short int *)s.nimages);
     level->squares[i].delay = ntohs(*(unsigned short *)s.delay);
     blksz = ntohs(*(unsigned short *)s.filenamesz);
     trace("Square: %d",i);
     trace("Properties: %.4x",level->squares[i].properties);
     trace("Images: %d",level->squares[i].nimages);
     trace("Delay: %dms",level->squares[i].delay);
     images = malloc(blksz);
     if (images==NULL)
      {
	fprintf(stderr,"Not enough memory need to allocate (%d)\n",blksz);
	close(file);
	free(level);
	return NULL;
      }
     read(file,images,blksz);
     level->squares[i].images = malloc(sizeof(SDL_Surface *)* level->squares[i].nimages);
     if (level->squares[i].images==NULL)
      {
	fprintf(stderr,"Not enough memory need to allocate (%d)\n",sizeof(SDL_Surface *)* level->squares[i].nimages);
	close(file);
	free(level);
	free(images);
	return NULL;
      }
     memset(level->squares[i].images,0,sizeof(SDL_Surface *)* level->squares[i].nimages);
     for (j=0,p=images;j<level->squares[i].nimages;j++)
      {
	SDL_Surface *image = Square_Load(p);
	level->squares[i].images[j] = image;
	if (level->squares[i].images[j]==NULL)
	 {
	   fprintf(stderr,"Can't read graphics square (%s)\n",p);
	   Square_Free(&(level->squares[i]));
	   close(file);
	   free(level);
	   free(images);
	   return NULL;
	 }
	while (*p)
	  p++;
	p++;
      }
     free(images);
   }
  /* Load the data map */
  level->map = malloc(level->sizex*level->sizey*sizeof(int));
  if (level->map==NULL)
   {
     close(file);
     for (i=0;i<nsquares;i++)
       Square_Free(&(level->squares[i]));
     free(level);
     return NULL;
   }
  for (i=0;i<level->sizex*level->sizey;i++)
   {
     if (nsquares<256)
      {
	unsigned char value;
	read(file,&value,1);
	level->map[i]=value;
      }
     else
      {
	unsigned short int value;
	read(file,&value,2);
	value = htons(*(unsigned short int *)&value);
	level->map[i]=value;
      }
   }
  close(file);

  /* Find the start and finish square */
  map = level->map;
  level->start_x=level->start_y=level->finish_x=level->finish_y=-1;
  for (y=0;y<level->sizey;y++)
   {
     for (x=0;x<level->sizex;x++)
      {
	int square_index = *map++;
	if (square_index)
	 {
	   if (level->squares[square_index].properties&START)
	    {
	      level->xdino = level->start_x=x;
	      level->ydino = level->start_y=y;

	    }
	   else if (level->squares[square_index].properties&FINISH)
	    {
	      /* TODO: change this to use a list, we can have more than one exit */
	      level->finish_x=x;
	      level->finish_y=y;
	    }
	 }
      }
   }

  /* Square 0 is special. We load all sprite we need */
  level->squares[0].nimages = 2;
  level->squares[0].images = malloc(sizeof(SDL_Surface *)* level->squares[0].nimages);
  if (level->squares[0].images==NULL)
   { /* FIXME: free allocated memory */
     fprintf(stderr,"Not enough memory need to allocate (%d)\n",sizeof(SDL_Surface *)* level->squares[0].nimages);
     return NULL;
   }

  level->squares[0].images[0] = Image_Load("squares/dinosaur.png");
  if (level->squares[0].images[0] == NULL)
   { /* FIXME: free allocated memory */
     fprintf(stderr,"Can't read the dinosaur sprite (squares/dinosaur.png)\n");
     return NULL;
   }

  level->squares[0].images[1] = Image_Load("squares/kinesist.png");
  if (level->squares[0].images[1] == NULL)
   { /* FIXME: free allocated memory */
     fprintf(stderr,"Can't read the kinesist sprite (squares/kinesist.png)\n");
     return NULL;
   }


  /* Some default variable for a level */
  level->control=CONTROL_PLAYER;

  return level;
}


struct t_level *Level_Load(char *filename)
{
  int i;
  char tmp[1024];
  struct t_level *l=NULL;
  char *datadir_path[]=
   {
     STONAGE_DATA_DIR,
     ".",
     "../data",
     NULL
   };

  i=0;
  while (datadir_path[i])
   {
     snprintf(tmp,1024,"%s/levels/%s",datadir_path[i],filename);
     trace("Trying to load %s",tmp);
     l = Level_Load_Real(tmp);
     if (l)
       break;
     i++;
   }
  if (datadir_path[i]==NULL)
   {
     fprintf(stderr,"Can't load this level %s\n",filename);
     return NULL;
   }
  return l;
}

int Level_Sanity_Check(struct t_level *l)
{
  int ok = 1;
  int i;
  int *map;

  /* Check that square[x][y]<nsquares */
  map = l->map;
  for (i=0;i< (l->sizex*l->sizey);i++,map++)
   {
     if (*map >= l->nsquares)
      {
	trace("x=%d y=%d val=%d",i%l->sizex,i/l->sizex,*map);
	fprintf(stderr,"This level doesn't pass the sanity check.\nAborting\n");
	return 0;
      }
   }

  if (l->start_x==-1 || l->start_y==-1)
   {
     fprintf(stderr,"This level doesn't have a starting point.\nAborting\n");
     return 0;
   }

  if (l->finish_x==-1 || l->finish_y==-1)
   {
     fprintf(stderr,"This level doesn't have at least a ending point.\nAborting\n");
     return 0;
   }

  return ok;
}



int Level_Draw2Screen(struct t_level *l,SDL_Surface *screen)
{
  int x,y,square_index,xbegin;
  SDL_Rect dest;
  SDL_Surface *image;
  int *map = l->map;

  /* Compute some values */

  /* All squares have the same size (for now) */
  dest.w = l->squares[1].images[0]->w;
  dest.h = l->squares[1].images[0]->h;

  xbegin = (screen->w - (dest.w * l->sizex))/2;
  dest.y = (screen->h - (dest.h * l->sizey))/2;
  
  /* Draw the background */
  if (l->background==NULL)
   {
     /* Fill the backcolor in black */
     SDL_FillRect(screen,NULL,0);
   }
  else
   {
     SDL_BlitSurface(l->background, NULL, screen, NULL);
   }
  /* Draw the playfield */
  for (y=0;y<l->sizey;y++)
   {
     dest.x = xbegin;
     for (x=0;x<l->sizex;x++)
      {
	square_index = *map++;
	if (square_index)
	 {
	   image = l->squares[square_index].images[0];
	   SDL_BlitSurface(image, NULL, screen, &dest);
	 }

	dest.x +=dest.w;
      }
     dest.y+=dest.h;
   }
  /* Draw the dinosaur */
  dest.x = xbegin + (dest.w * l->xdino);
  dest.y = ((screen->h - (dest.h * l->sizey))/2) + (dest.h * l->ydino);
  SDL_BlitSurface(l->squares[0].images[0], NULL, screen, &dest);

  /* ... and draw the kinesist sprite */
  if (l->control==CONTROL_KINESIST ||
      l->control==CONTROL_SQUARE)
   {
     dest.x = xbegin + (dest.w * l->xkinesist);
     dest.y = ((screen->h - (dest.h * l->sizey))/2) + (dest.h * l->ykinesist);
     SDL_BlitSurface(l->squares[0].images[1], NULL, screen, &dest);
   }

  return 1;
}


/*
 * Function: Level_TestCanGoAbove
 * Comment:
 *   We test if the dinausor can go abose the square, we make a function,
 *   because we can test if the user have the good key, and all other actions.
 * INPUT:
 * 
 * OUTPUT:
 *  0: can't move
 *  1: ok.
 */

static int
Level_TestCanGoAbove(struct t_level *l, unsigned int square_properties)
{
  if (   (square_properties&CAN_GO_ABOVE)
      && !((square_properties&NEED_KEY_RED) && !(l->keys&KEY_RED))
      && !((square_properties&NEED_KEY_BLUE) && !(l->keys&KEY_BLUE))
      && !((square_properties&NEED_KEY_GREEN) && !(l->keys&KEY_GREEN)))
   {
     return 1;
   }
  return 0;
}

/*
 * Function: Level_MoveDinoTo
 * INPUT:
 * direction: 0 down, 1:up, 2:right, 3:left
 * OUTPUT:
 *  0: can't move
 *  1: ok.
 */

int
Level_MoveDinoTo(struct t_level *l, enum movement_type direction)
{
  int square_index, square_moved, x, y;
  unsigned int properties;
  int *map;
  /* Some sanity check over buffer overflow */
  if (  (direction==MOVE_RIGHT && l->xdino >= l->sizex) 
      ||(direction==MOVE_LEFT  && l->xdino == 0) 
      ||(direction==MOVE_DOWN  && l->ydino >= l->sizey) 
      ||(direction==MOVE_UP    && l->ydino == 0) )
   {
     return 0;
   }
  
  map = l->map + (l->ydino * l->sizex) + l->xdino;
  trace("direction=%d",direction);
  switch(direction)
   {
     /* Each test is done in that order:
      * we test if the dinosaur can go in the good direction,
      * we test if the square where the dinosaur is can move
      */
    case MOVE_RIGHT:
      square_index = *(map+1);
      properties = l->squares[square_index].properties;
      if (Level_TestCanGoAbove(l,properties))
       {
	 l->xdino++;
	 return 1;
       }

      square_index = *map;
      properties = l->squares[square_index].properties;
      if (properties&CAN_MOVE_TO_EAST)
       {
	 /* Ok move the square then move the dinosaur */
	 x=l->xdino;
	 square_moved=*map;
	 *map=0;
	 while (x<l->sizex-1 && *(map+1)==0)
	  {
	    x++; map++;
	  }
	 l->xdino=x;
	 *map=square_moved;
	 return 1;
       }

      return 0;

    case MOVE_LEFT:
      square_index = *(map-1);
      properties = l->squares[square_index].properties;
      if (Level_TestCanGoAbove(l,properties))
       {
	 l->xdino--;
	 return 1;
       }

      square_index = *map;
      properties = l->squares[square_index].properties;
      if (properties&CAN_MOVE_TO_WEST)
       {
	 /* Ok move the square then move the dinosaur */
	 x=l->xdino;
	 square_moved=*map;
	 *map=0;
	 while (x>0 && *(map-1)==0)
	  {
	    x--; map--;
	  }
	 l->xdino=x;
	 *map=square_moved;
	 return 1;
       }

      return 0;

    case MOVE_UP:
      square_index = *(map - l->sizex);
      properties = l->squares[square_index].properties;
      if (Level_TestCanGoAbove(l,properties))
       {
	 l->ydino--;
	 return 1;
       }

      square_index = *map;
      properties = l->squares[square_index].properties;
      if (properties&CAN_MOVE_TO_NORTH)
       {
	 /* Ok move the square then move the dinosaur */
	 y=l->ydino;
	 square_moved=*map;
	 *map=0;
	 while (y>0 && *(map - l->sizex)==0)
	  {
	    y--; map -= l->sizex;
	  }
	 l->ydino=y;
	 *map=square_moved;
	 return 1;
       }

      return 0;

    case MOVE_DOWN:
      square_index = *(map + l->sizex);
      properties = l->squares[square_index].properties;
      if (Level_TestCanGoAbove(l,properties))
       {
	 l->ydino++;
	 return 1;
       }

      square_index = *map;
      properties = l->squares[square_index].properties;
      if (properties&CAN_MOVE_TO_SOUTH)
       {
	 /* Ok move the square then move the dinosaur */
	 y=l->ydino;
	 square_moved=*map;
	 *map=0;
	 while (y<l->sizey-1 && *(map + l->sizex)==0)
	  {
	    y++; map += l->sizex;
	  }
	 l->ydino=y;
	 *map=square_moved;
	 return 1;
       }

      return 0;

    default:
      return 0;
   }
}

/*
 * Function: Level_MoveKinesistTo
 * INPUT:
 * direction: 0 down, 1:up, 2:right, 3:left
 * OUTPUT:
 *  0: can't move
 *  1: ok.
 */
int
Level_MoveKinesistTo(struct t_level *l, enum movement_type direction)
{
  int *map;
  /* Some sanity check over buffer overflow */
  if (  (direction==MOVE_RIGHT && l->xkinesist >= l->sizex-1)
      ||(direction==MOVE_LEFT  && l->xkinesist == 0)
      ||(direction==MOVE_DOWN  && l->ykinesist >= l->sizey-1)
      ||(direction==MOVE_UP    && l->ykinesist == 0))
   {
     return 0;
   }
  
  map = l->map + (l->ykinesist * l->sizex) + l->xkinesist;
  trace("direction=%d",direction);
  switch(direction)
   {
    case MOVE_RIGHT:
      l->xkinesist++;
      return 1;

    case MOVE_LEFT:
      l->xkinesist--;
      return 1;

    case MOVE_UP:
      l->ykinesist--;
      return 1;

    case MOVE_DOWN:
      l->ykinesist++;
      return 1;

    default:
      return 0;
   }
}

/*
 * Function: Level_CanMoveSquare
 * Comment:
 *   Return True when the user can move this square with the telekinesist action
 *   If the square can only move in one direction, move the square.
 * Input:
 *
 * Output:
 */
int
Level_CanMoveSquare(struct t_level *l)
{
  int square_can_move;
  unsigned int properties;
  int *map;

  map = l->map + (l->ykinesist * l->sizex) + l->xkinesist;
  properties = l->squares[*map].properties;

  /* If the dinausor is on the square, don't move this square */
  if ((l->xkinesist == l->xdino) && (l->ykinesist == l->ydino))
    return 0;

  square_can_move=0;

  properties&=CAN_MOVE_TO_MASK;
  if (!properties) /* If the square can't move, exit now */
    return 0;

  /* For each direction,
   *  test if the square can move again,
   *   and if it's the only direction, move it
   */
  if (properties&CAN_MOVE_TO_WEST)
   {
     if (*(map - 1)==0)
      {
	square_can_move=1;
	if (properties == CAN_MOVE_TO_WEST)
	  return Level_MoveSquareTo(l,MOVE_LEFT);
      }
   }
  
  if (properties&CAN_MOVE_TO_EAST)
   {
     if (*(map + 1)==0)
      {
	square_can_move=1;
	if (properties == CAN_MOVE_TO_EAST)
	  return Level_MoveSquareTo(l,MOVE_RIGHT);
      }
   }

  if (properties&CAN_MOVE_TO_NORTH)
   {
     if (*(map - l->sizex)==0)
      {
	square_can_move=1;
	if (properties == CAN_MOVE_TO_NORTH)
	  return Level_MoveSquareTo(l,MOVE_UP);
      }
   }
  
  if (properties&CAN_MOVE_TO_SOUTH)
   {
     if (*(map + l->sizex)==0)
      {
	square_can_move=1;
	if (properties == CAN_MOVE_TO_SOUTH)
	  return Level_MoveSquareTo(l,MOVE_DOWN);
      }
   }

  return square_can_move;
}

/*
 * Function: Level_MoveSquare
 * Comment:
 *   Move the square in the direction indicated
 * Input:
 *
 * Output:
 */
int
Level_MoveSquareTo(struct t_level *l, enum movement_type direction)
{
  int x,y,square_moved,properties;
  int *map;

  map = l->map + (l->ykinesist * l->sizex) + l->xkinesist;
  properties = l->squares[*map].properties;

  switch(direction)
   {
    case MOVE_RIGHT:
      if (properties&CAN_MOVE_TO_EAST)
       {
	 x=l->xkinesist;
	 square_moved=*map;
	 *map=0;
	 while (x<l->sizex-1 && *(map+1)==0)
	  {
	    x++; map++;
	  }
	 l->xkinesist=x;
	 *map=square_moved;
	 return 1;
       }
      return 0;

    case MOVE_LEFT:
      if (properties&CAN_MOVE_TO_WEST)
       {
	 x=l->xkinesist;
	 square_moved=*map;
	 *map=0;
	 while (x>0 && *(map-1)==0)
	  {
	    x--; map--;
	  }
	 l->xkinesist=x;
	 *map=square_moved;
	 return 1;
       }
      return 0;

    case MOVE_UP:
      if (properties&CAN_MOVE_TO_NORTH)
       {
	 y=l->ykinesist;
	 square_moved=*map;
	 *map=0;
	 while (y>0 && *(map - l->sizex)==0)
	  {
	    y--; map -= l->sizex;
	  }
	 l->ykinesist=y;
	 *map=square_moved;
	 return 1;
       }
      return 0;

    case MOVE_DOWN:
      if (properties&CAN_MOVE_TO_SOUTH)
       {
	 y=l->ykinesist;
	 square_moved=*map;
	 *map=0;
	 while (y<l->sizey-1 && *(map + l->sizex)==0)
	  {
	    y++; map += l->sizex;
	  }
	 l->ykinesist=y;
	 *map=square_moved;
	 return 1;
       }
      return 0;
   }

 return 0;
}

/*
 * Function: Level_DoPostMoveAction
 * Comment:
 *   Execute function after the move and before the animation.
 * Input:
 *
 * Output:
 *   TRUE or FALSE
 */
int
Level_DoPostMoveAction(struct t_level *l)
{
  int square_index;
  unsigned int properties;
  
  square_index=l->map[ (l->ydino * l->sizex) + l->xdino];
  properties = l->squares[square_index].properties;

  if ((properties&KEY_RED) || (properties&NEED_KEY_RED))
   {
     l->keys|=KEY_RED;
     l->map[ (l->ydino * l->sizex) + l->xdino] = 2; /* FIXME: We must find the correct index */
   }
  if ((properties&KEY_GREEN) || (properties&NEED_KEY_GREEN))
   {
     l->keys|=KEY_GREEN;
     l->map[ (l->ydino * l->sizex) + l->xdino] = 2; /* FIXME: We must find the correct index */
   }
  if ((properties&KEY_BLUE) || (properties&NEED_KEY_BLUE))
   {
     l->keys|=KEY_BLUE;
     l->map[ (l->ydino * l->sizex) + l->xdino] = 2; /* FIXME: We must find the correct index */
   }
  if (properties&AFTER_MOVE_CRASH)
   {
     l->map[ (l->ydino * l->sizex) + l->xdino] = 0;
   }


  return 0;
}


/*
 * Function: Level_IsFinish
 * Comment:
 *   return true when the user have accomplished all tasks. For now,
 *   we only test if the user is on a Finish square
 * Input:
 *
 * Output:
 *   TRUE or FALSE
 */
int
Level_IsFinish(struct t_level *l)
{
  int square_index;
  unsigned int properties;
  
  square_index=l->map[ (l->ydino * l->sizex) + l->xdino];
  properties = l->squares[square_index].properties;

  if (properties&FINISH)
    return 1;
  else
    return 0;
}


