/*
 * Source for loading/drawing squre
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif
#include "square.h"
#include "trace.h"
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <SDL.h>
#if defined(HAVE_SDL_SDL_IMAGE_H)
#include <SDL/SDL_image.h>
#endif

static SDL_Surface *Square_Load_Real(char *filename);


static SDL_Surface *Square_Load_Real(char *filename)
{
  SDL_Surface *image, *image2;

#if defined(HAVE_SDL_SDL_IMAGE_H)
  image = IMG_Load(filename);
#else
  image = SDL_LoadBMP(filename);
#endif
  if (image==NULL)
    return NULL;
  image2 = SDL_DisplayFormat(image);
  SDL_FreeSurface(image);
  return image2;
}

SDL_Surface *
Square_Load(char *filename)
{
  int i;
  char tmp[1024];
  SDL_Surface *surface=NULL;
  char *datadir_path[]=
   {
     STONAGE_DATA_DIR,
     ".",
     "../data",
     NULL
   };

  i=0;
  while (datadir_path[i])
   {
     snprintf(tmp,1024,"%s/squares/%s",datadir_path[i],filename);
     trace("Trying to load %s",tmp);
     surface = Square_Load_Real(tmp);
     if (surface)
       break;
     i++;
   }
  if (datadir_path[i]==NULL)
   {
     fprintf(stderr,"Can't load this square %s\n",filename);
     return NULL;
   }
  return surface;
}

/* We don't free the structure itself */
void Square_Free(struct t_square *s)
{
  int i;
  if (s!=NULL)
   {
     if (s->images!=NULL)
      {
	for (i=0;i<s->nimages;i++)
	  SDL_FreeSurface(s->images[i]);
	free(s->images);
      }
   }
}

#if 0
/*
 * Functions to parse the squarefile
 */
static void jump_space(unsigned char **ptr)
{
  char *p=*ptr;
  while (*p==' ' || *p=='\t' || *p=='\n')
    p++;
  if (*(p-1)=='\n')
    p--;
  *ptr=p;
}

static void jump_word(unsigned char **ptr)
{
  char *p=*ptr;
  while (*p!=' ' && *p!='\t' && *p!='\n')
    p++;
  *ptr=p;
}

static void jump_line(unsigned char **ptr)
{
  char *p=*ptr;
  while (*p!='\n')
    p++;
  *ptr=++p;
}

static int if_yes(unsigned char **ptr, int flag)
{
  if (**ptr=='Y' || **ptr=='1' || **ptr=='J' || **ptr=='O')
   {
     jump_word(ptr);
     jump_space(ptr);
     return flag;
   }
  else if (**ptr=='N' || **ptr=='0')
   {
     jump_word(ptr);
     jump_space(ptr);
     return 0;
   }
  else
   {
     fprintf(stderr,"Warning boolean value unknow: %c\n",**ptr);
     jump_word(ptr);
     jump_space(ptr);
     return 0;
   }
}

struct t_square *Square_Load(const char *filename)
{
  struct t_square *s;
  unsigned char *buffer=NULL, *ptr;
  int len;
  int nsquare;

  /* Allocation des graphiques pour le level */
  s=(struct t_square *)malloc(sizeof(struct t_square)*64);
  if (!s)
    return NULL;
  /* Chargement du fichier */
  len=Read_File((char *)filename,(void **)&buffer);
  if (len==0)
  {
    if (buffer!=NULL)
      free(buffer);
    free(s);
    return NULL;
  }
  /* Parse du fichier en mmoire */
  ptr=buffer;
  nsquare=0;
  while (ptr<buffer+len)
   {
     if ((*ptr=='#') ||/* Est ce que c'est un commentaire ? */
	 (*ptr=='\n')) /* Est ce que c'est une ligne vide ? */
      {
	jump_line(&ptr);
      }
     else /* C'est une ligne valide, chaque paramtre est spar par un ou plusieurs espaces */
      {
	char *file=ptr;
	int flags=0;
	jump_word(&ptr);
	*ptr++=0;
	jump_space(&ptr);
	flags|=if_yes(&ptr,CAN_MOVE_TO_NORTH);
	flags|=if_yes(&ptr,CAN_MOVE_TO_SOUTH);
	flags|=if_yes(&ptr,CAN_MOVE_TO_EAST);
	flags|=if_yes(&ptr,CAN_MOVE_TO_WEST);
	flags|=if_yes(&ptr,AFTER_MOVE_CRASH);
	flags|=if_yes(&ptr,CAN_GO_ABOVE);

	s[nsquare].flags=flags;
#ifdef DEBUG
	s[nsquare].file=file;
#endif

	/* Chargement de l'image */
	if (Image_TGAFileRead(&(s[nsquare].image),file)==0)
	 {
	   printf("Error: Can't load this image file (%s)\n",file);
	 }
	else
	 {
	   Square_Print(s+nsquare);
	   nsquare++;
	 }
	jump_line(&ptr);
      }
   }

  return s;
}

void Square_Print(struct t_square *s)
{
#ifdef DEBUG
  printf("%20s  N S E W C A\n",s->file);
#else
  printf("%20s  N S E W C A\n","");
#endif
  printf("                      ");
  printf(((s->flags)&CAN_MOVE_TO_NORTH)?"Y ":"N ");
  printf(((s->flags)&CAN_MOVE_TO_SOUTH)?"Y ":"N ");
  printf(((s->flags)&CAN_MOVE_TO_EAST)?"Y ":"N ");
  printf(((s->flags)&CAN_MOVE_TO_WEST)?"Y ":"N ");
  printf(((s->flags)&AFTER_MOVE_CRASH)?"Y ":"N ");
  printf(((s->flags)&CAN_GO_ABOVE)?"Y ":"N ");

  printf("\n");
}

#endif


